@@ -1,27 +1,63 @@
+
+
+
+/*
+ Authors 
+ Martin Schlather, schlather@math.uni-mannheim.de
+
+
+ Copyright (C) 2015 Martin Schlather
+
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation; either version 3
+of the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.  
+*/
+
+
 #ifndef RFerror_H
 #define RFerror_H 1
 
+extern char MSG[LENERRMSG], BUG_MSG[250], MSG2[LENERRMSG],
+  ERRORSTRING[MAXERRORSTRING], ERRORSTRING_OK[MAXERRORSTRING],
+  ERRORSTRING_WRONG[MAXERRORSTRING], ERROR_LOC[nErrorLoc];
+extern int ERRORMODELNUMBER;
+void errorMSG(int error, char* EM);
+void FinalErrorMSG(int err, char* m);
+void ErrorStop(int err);
 
 // Error codes & messages
 // positive values are error codes
 // negative values are messages
 
-#define NOERROR 0                 
-#define ERRORNOTPROGRAMMEDYET 1     
-#define ERRORFAILED 2           /* method didn't work for the specified 
-				    parameters */
-#define ERRORVDIMNOTPROGRAMMEDYET 3
-#define ERRORTYPECONSISTENCY 4
+// #define NOERROR 0                 
+// #define ERRORMEMORYALLOCATION 1 /* NEVER CHANGE VALUE; MALLOC returned NULL pointer */
+
+
+
+// LESS THAN 10 IS RESERVED TO BE JOINTLY USED BY RFUTILS
+
 
-#define ERRORM 10                /* a single error message */
 #define ERRORMEND 11         /* a single error message -- und alles dazwischen */
+#define ERRORCARTESIAN 12
 #define ERRORWRONG 13           /* same as ERRORMSG, but showing only
 				   ERRORSTRING_WRONG */
 
+#define ERRORTYPECONSISTENCY 14
 #define ERRORWRONGVDIM 15
 #define ERRORBADVDIM 16
 #define ERRORNOTCARTESIAN 17
 #define ERRORODDCOORDTRAFO 18
+#define ERRORVDIMNOTPROGRAMMEDYET 19
 
 #define ERRORNOTDEFINED 20       /* the specification for the  covariance and 
 				    method is not given/known, e.g. TBM2 for 
@@ -35,7 +71,9 @@
 #define ERRORANISO_T 24         /* anisoT' may not be given at the same
 				time with 'Aniso' or 'proj' */
 //#define ERRORVARIOGRAMONLY 25    /* attempt to get the covariance whereas only 				    the variogram is defined */
+#define ERRORDECOMPOSITION 26   /* direct.cc */
 #define ERRORPREFNONE 27
+#define ERRORPREFNONECOV 28
 
 #define ERRORSUBMETHODFAILED 30 /* aufsplitten der Modelle bringt nichts */
 #define ERRORMAXDIMMETH 31     /* max dimension of method exceeded */
@@ -53,7 +91,6 @@
                                    appropriate for the respective method */
 #define ERRORTOOMANYLINES 41    /* Hyperplane tesselation: estimated simulated
 				   lines passes given threshold*/
-#define ERRORDECOMPOSITION 42   /* direct simulation; matrix inversion failed */
 #define ERRORNOVARIOGRAM 43     /* variogram models not allowed in 
 				   ci and direct */
 #define ERRORSPECTRAL 44
@@ -81,9 +118,10 @@
 #define ERROR_MATRIX_SQUARE 103
 #define ERROR_MATRIX_VDIM 104
 #define ERROR_MATRIX_POSDEF 105
+#define MATRIX_NOT_CHECK_YET -999  /* must be a negative value not used somewhere else ! */
+
 //#define ERROR_MATRIX_ 10
 
-#define ERRORMEMORYALLOCATION 106 /* MALLOC returned NULL pointer */
 #define ERRORNOTINITIALIZED 107   /* key.active==false in DoSimulateRF; is only 
 				    checked there !!! */
 #define ERRORDIM 119              /* dim<1 or dim>MAXDIM */
@@ -119,9 +157,12 @@
 #define MSGLOCAL_SIGNPHISND 406
 #define MSGLOCAL_INITINTRINSIC 407
 #define MSGLOCAL_FAILED 408
+#define MSGLOCAL_NOPOSITIVEROOT 409
 #define ERRORUNSPECIFIED 999  
 
 /* do not use numbers 800 -- 900 : reserved to MPP package */
 
+
+
 #endif
 
